import 'package:diameter/components/detail.dart';
import 'package:diameter/components/dialogs.dart';
import 'package:diameter/components/dropdown.dart';
import 'package:diameter/components/forms.dart';
import 'package:diameter/models/accuracy.dart';
import 'package:diameter/models/log_meal.dart';
import 'package:diameter/models/meal.dart';
import 'package:diameter/models/meal_category.dart';
import 'package:diameter/models/meal_portion_type.dart';
import 'package:diameter/models/meal_source.dart';
import 'package:diameter/models/settings.dart';
import 'package:diameter/navigation.dart';
import 'package:diameter/screens/accuracy_detail.dart';
import 'package:diameter/screens/meal/meal_category_detail.dart';
import 'package:diameter/screens/meal/meal_detail.dart';
import 'package:diameter/screens/meal/meal_portion_type_detail.dart';
import 'package:diameter/screens/meal/meal_source_detail.dart';
import 'package:diameter/utils/utils.dart';
import 'package:flutter/material.dart';

class LogMealDetailScreen extends StatefulWidget {
  static const String routeName = '/log-meal';

  final int logEntryId;
  final int id;

  const LogMealDetailScreen({Key? key, this.logEntryId = 0, this.id = 0})
      : super(key: key);

  @override
  _LogMealDetailScreenState createState() => _LogMealDetailScreenState();
}

class _LogMealDetailScreenState extends State<LogMealDetailScreen> {
  LogMeal? _logMeal;
  bool _isNew = true;
  bool _isSaving = false;
  bool _isExpanded = false;

  final GlobalKey<FormState> _logMealForm = GlobalKey<FormState>();
  final ScrollController _scrollController = ScrollController();

  final _valueController = TextEditingController(text: '');
  final _carbsRatioController = TextEditingController(text: '');
  final _portionSizeController = TextEditingController(text: '');
  final _carbsPerPortionController = TextEditingController(text: '');
  final _notesController = TextEditingController(text: '');

  Meal? _meal;
  MealSource? _mealSource;
  MealCategory? _mealCategory;
  MealPortionType? _mealPortionType;
  Accuracy? _portionSizeAccuracy;
  Accuracy? _carbsRatioAccuracy;

  final _mealController = TextEditingController(text: '');
  final _mealSourceController = TextEditingController(text: '');
  final _mealCategoryController = TextEditingController(text: '');
  final _mealPortionTypeController = TextEditingController(text: '');
  final _portionSizeAccuracyController = TextEditingController(text: '');
  final _carbsRatioAccuracyController = TextEditingController(text: '');

  List<Meal> _meals = [];
  List<MealCategory> _mealCategories = [];
  List<MealPortionType> _mealPortionTypes = [];
  List<MealSource> _mealSources = [];
  List<Accuracy> _portionSizeAccuracies = [];
  List<Accuracy> _carbsRatioAccuracies = [];

  @override
  void initState() {
    super.initState();
    reload();

    _portionSizeAccuracies = Accuracy.getAllForPortionSize();
    _carbsRatioAccuracies = Accuracy.getAllForCarbsRatio();
    _meals = Meal.getAll();
    _mealCategories = MealCategory.getAll();
    _mealPortionTypes = MealPortionType.getAll();
    _mealSources = MealSource.getAll();

    if (widget.id != 0) {
      _valueController.text = _logMeal!.value;
      _carbsRatioController.text = (_logMeal!.carbsRatio ?? '').toString();
      _portionSizeController.text = (_logMeal!.portionSize ?? '').toString();
      _carbsPerPortionController.text =
          (_logMeal!.carbsPerPortion ?? '').toString();
      _notesController.text = _logMeal!.notes ?? '';

      _meal = _logMeal!.meal.target;
      _mealController.text = (_meal ?? '').toString();
      _mealSource = _logMeal!.mealSource.target;
      _mealSourceController.text = (_mealSource ?? '').toString();
      _mealCategory = _logMeal!.mealCategory.target;
      _mealCategoryController.text = (_mealCategory ?? '').toString();
      _mealPortionType = _logMeal!.mealPortionType.target;
      _mealPortionTypeController.text = (_mealPortionType ?? '').toString();
      _portionSizeAccuracy = _logMeal!.portionSizeAccuracy.target;
      _portionSizeAccuracyController.text =
          (_portionSizeAccuracy ?? '').toString();
      _carbsRatioAccuracy = _logMeal!.carbsRatioAccuracy.target;
      _carbsRatioAccuracyController.text =
          (_carbsRatioAccuracy ?? '').toString();
    }
  }

  void reload({String? message}) {
    if (widget.id != 0) {
      setState(() {
        _logMeal = LogMeal.get(widget.id);
      });
    }
    _isNew = _logMeal == null;

    setState(() {
      if (message != null) {
        var snackBar = SnackBar(
          content: Text(message),
          duration: const Duration(seconds: 2),
        );
        ScaffoldMessenger.of(context)
          ..removeCurrentSnackBar()
          ..showSnackBar(snackBar);
      }
    });
  }

  void updateCarbsRatioAccuracy(Accuracy? value) {
    setState(() {
      _carbsRatioAccuracy = value;
      _carbsRatioAccuracyController.text =
          (_carbsRatioAccuracy ?? '').toString();
    });
  }

  void updatePortionSizeAccuracy(Accuracy? value) {
    setState(() {
      _portionSizeAccuracy = value;
      _portionSizeAccuracyController.text =
          (_portionSizeAccuracy ?? '').toString();
    });
  }

  void updateMealCategory(MealCategory? value) {
    setState(() {
      _mealCategory = value;
      _mealCategoryController.text = (_mealCategory ?? '').toString();
    });
  }

  void updateMealPortionType(MealPortionType? value) {
    setState(() {
      _mealPortionType = value;
      _mealPortionTypeController.text = (_mealPortionType ?? '').toString();
    });
  }

  void updateMealSource(MealSource? value) {
    setState(() {
      _mealSource = value;
      _mealSourceController.text = (_mealSource ?? '').toString();
    });
  }

  Future<void> onSelectMeal(Meal? meal) async {
    setState(() {
      _meal = meal;
      _valueController.text = _mealController.text = (_meal ?? '').toString();
    });

    if (meal != null) {
      if (meal.carbsRatio != null) {
        _carbsRatioController.text = meal.carbsRatio.toString();
      }
      if (meal.portionSize != null) {
        _portionSizeController.text = meal.portionSize.toString();
      }
      if (meal.carbsPerPortion != null) {
        _carbsPerPortionController.text = meal.carbsPerPortion.toString();
      }
      if (meal.mealSource.hasValue) {
        updateMealSource(meal.mealSource.target);
      }
      if (meal.mealCategory.hasValue) {
        updateMealCategory(meal.mealCategory.target);
      }
      if (meal.mealPortionType.hasValue) {
        updateMealPortionType(meal.mealPortionType.target);
      }
      if (meal.portionSizeAccuracy.hasValue) {
        updatePortionSizeAccuracy(meal.portionSizeAccuracy.target);
      }
      if (meal.carbsRatioAccuracy.hasValue) {
        updateCarbsRatioAccuracy(meal.carbsRatioAccuracy.target);
      }
    }
  }

  void handleSaveAction() async {
    setState(() {
      _isSaving = true;
    });
    if (_logMealForm.currentState!.validate()) {
      LogMeal logMeal = LogMeal(
        id: widget.id,
        value: _valueController.text,
        carbsRatio: double.tryParse(_carbsRatioController.text),
        portionSize: double.tryParse(_portionSizeController.text),
        carbsPerPortion: double.tryParse(_carbsPerPortionController.text),
      );
      logMeal.logEntry.targetId = widget.logEntryId;
      logMeal.meal.target = _meal;
      logMeal.mealSource.target = _mealSource;
      logMeal.mealCategory.target = _mealCategory;
      logMeal.mealPortionType.target = _mealPortionType;
      logMeal.portionSizeAccuracy.target = _portionSizeAccuracy;
      logMeal.carbsRatioAccuracy.target = _carbsRatioAccuracy;

      LogMeal.put(logMeal);
      Navigator.pop(context, ['${_isNew ? 'New' : ''} Meal Saved', logMeal]);
    }
    setState(() {
      _isSaving = false;
    });
  }

  void handleCancelAction() {
    if (Settings.get().showConfirmationDialogOnCancel &&
        ((_isNew &&
                (_valueController.text != '' ||
                    _meal != null ||
                    _mealSource != null ||
                    _mealCategory != null ||
                    _mealPortionType != null ||
                    double.tryParse(_carbsRatioController.text) != null ||
                    double.tryParse(_portionSizeController.text) != null ||
                    double.tryParse(_carbsPerPortionController.text) != null ||
                    _carbsRatioAccuracy != null ||
                    _portionSizeAccuracy != null ||
                    _notesController.text != '')) ||
            (!_isNew &&
                (_valueController.text != _logMeal!.value ||
                    _meal != _logMeal!.meal.target ||
                    _mealSource != _logMeal!.mealSource.target ||
                    _mealCategory != _logMeal!.mealCategory.target ||
                    _mealPortionType != _logMeal!.mealPortionType.target ||
                    double.tryParse(_carbsRatioController.text) !=
                        _logMeal!.carbsRatio ||
                    double.tryParse(_portionSizeController.text) !=
                        _logMeal!.portionSize ||
                    double.tryParse(_carbsPerPortionController.text) !=
                        _logMeal!.carbsPerPortion ||
                    _carbsRatioAccuracy !=
                        _logMeal!.carbsRatioAccuracy.target ||
                    _portionSizeAccuracy !=
                        _logMeal!.portionSizeAccuracy.target ||
                    _notesController.text != (_logMeal!.notes ?? ''))))) {
      Dialogs.showCancelConfirmationDialog(
        context: context,
        isNew: _isNew,
        onSave: handleSaveAction,
      );
    } else {
      Navigator.pop(context);
    }
  }

  void calculateThirdMeasurementOfPortionCarbsRelation(
      {PortionCarbsParameter? parameterToBeCalculated}) {
    double? carbsRatio;
    double? portionSize;
    double? carbsPerPortion;

    if (parameterToBeCalculated != PortionCarbsParameter.carbsRatio &&
        _carbsRatioController.text != '') {
      carbsRatio = double.tryParse(_carbsRatioController.text);
    }
    if (parameterToBeCalculated != PortionCarbsParameter.portionSize &&
        _portionSizeController.text != '') {
      portionSize = double.tryParse(_portionSizeController.text);
    }
    if (parameterToBeCalculated != PortionCarbsParameter.carbsPerPortion &&
        _carbsRatioController.text != '') {
      carbsPerPortion = double.tryParse(_carbsPerPortionController.text);
    }

    if (carbsRatio != null && portionSize != null && carbsPerPortion == null) {
      setState(() {
        _carbsPerPortionController.text =
            Utils.calculateCarbsPerPortion(carbsRatio!, portionSize!)
                .toString();
      });
    }
    if (carbsRatio == null && portionSize != null && carbsPerPortion != null) {
      setState(() {
        _carbsRatioController.text =
            Utils.calculateCarbsRatio(carbsPerPortion!, portionSize!)
                .toString();
      });
    }
    if (carbsRatio != null && portionSize == null && carbsPerPortion != null) {
      setState(() {
        _portionSizeController.text =
            Utils.calculatePortionSize(carbsRatio!, carbsPerPortion!)
                .toString();
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(_isNew ? 'New Meal' : _logMeal!.value),
      ),
      drawer: const Navigation(currentLocation: LogMealDetailScreen.routeName),
      body: Scrollbar(
        controller: _scrollController,
        child: SingleChildScrollView(
          controller: _scrollController,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.stretch,
            children: <Widget>[
              FormWrapper(
                formState: _logMealForm,
                fields: [
                  TextFormField(
                    controller: _valueController,
                    decoration: const InputDecoration(
                      labelText: 'Name',
                    ),
                    validator: (value) {
                      if (value!.trim().isEmpty) {
                        return 'Empty name';
                      }
                      return null;
                    },
                  ),
                  Row(
                    children: [
                      Expanded(
                        child: AutoCompleteDropdownButton<Meal>(
                          controller: _mealController,
                          selectedItem: _meal,
                          label: 'Meal',
                          items: _meals,
                          onChanged: (value) {
                            if (value != null) {
                              onSelectMeal(value);
                            }
                          },
                        ),
                      ),
                      IconButton(
                        onPressed: () {
                          Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (context) => _meal == null
                                  ? const MealDetailScreen()
                                  : MealDetailScreen(id: _meal!.id),
                            ),
                          ).then((result) {
                            onSelectMeal(result?[1]);
                            reload(message: result?[0]);
                          });
                        },
                        icon: Icon(_meal == null ? Icons.add : Icons.edit),
                      ),
                    ],
                  ),
                  Row(
                    children: [
                      Expanded(
                        child: TextFormField(
                          decoration: const InputDecoration(
                            labelText: 'Carbs ratio',
                            suffixText: '%',
                          ),
                          controller: _carbsRatioController,
                          keyboardType: const TextInputType.numberWithOptions(
                              decimal: true),
                          onChanged: (_) async {
                            await Future.delayed(const Duration(seconds: 1));
                            calculateThirdMeasurementOfPortionCarbsRelation();
                          },
                        ),
                      ),
                      IconButton(
                        onPressed: () =>
                            calculateThirdMeasurementOfPortionCarbsRelation(
                                parameterToBeCalculated:
                                    PortionCarbsParameter.carbsRatio),
                        icon: const Icon(Icons.calculate),
                      ),
                    ],
                  ),
                  Row(
                    mainAxisSize: MainAxisSize.min,
                    children: [
                      Expanded(
                        child: TextFormField(
                          decoration: InputDecoration(
                            labelText: 'Portion size',
                            suffixText: Settings.nutritionMeasurementSuffix,
                          ),
                          controller: _portionSizeController,
                          keyboardType: const TextInputType.numberWithOptions(
                              decimal: true),
                          onChanged: (_) async {
                            await Future.delayed(const Duration(seconds: 1));
                            calculateThirdMeasurementOfPortionCarbsRelation();
                          },
                        ),
                      ),
                      IconButton(
                        onPressed: () =>
                            calculateThirdMeasurementOfPortionCarbsRelation(
                                parameterToBeCalculated:
                                    PortionCarbsParameter.portionSize),
                        icon: const Icon(Icons.calculate),
                      ),
                    ],
                  ),
                  Row(
                    children: [
                      Expanded(
                        child: TextFormField(
                          decoration: InputDecoration(
                            labelText: 'Carbs per portion',
                            suffixText: Settings.nutritionMeasurementSuffix,
                          ),
                          controller: _carbsPerPortionController,
                          keyboardType: const TextInputType.numberWithOptions(
                              decimal: true),
                          onChanged: (_) async {
                            await Future.delayed(const Duration(seconds: 1));
                            calculateThirdMeasurementOfPortionCarbsRelation();
                          },
                        ),
                      ),
                      IconButton(
                        onPressed: () =>
                            calculateThirdMeasurementOfPortionCarbsRelation(
                                parameterToBeCalculated:
                                    PortionCarbsParameter.carbsPerPortion),
                        icon: const Icon(Icons.calculate),
                      ),
                    ],
                  ),
                  TextFormField(
                    controller: _notesController,
                    decoration: const InputDecoration(
                      labelText: 'Notes',
                    ),
                    keyboardType: TextInputType.multiline,
                    minLines: 2,
                    maxLines: 5,
                  ),
                  const Divider(),
                  GestureDetector(
                    onTap: () => setState(() {
                      _isExpanded = !_isExpanded;
                    }),
                    child: Row(
                      mainAxisSize: MainAxisSize.max,
                      children: [
                        Text(
                          'ADDITIONAL FIELDS',
                          style: Theme.of(context).textTheme.subtitle2,
                        ),
                        const Spacer(),
                        Icon(_isExpanded
                            ? Icons.expand_less
                            : Icons.expand_more),
                      ],
                    ),
                  ),
                  Column(
                    children: _isExpanded
                        ? [
                            Padding(
                              padding: const EdgeInsets.symmetric(vertical: 5.0),
                              child: Row(
                                children: [
                                  Expanded(
                                    child: AutoCompleteDropdownButton<MealSource>(
                                      controller: _mealSourceController,
                                      selectedItem: _mealSource,
                                      label: 'Meal Source',
                                      items: _mealSources,
                                      onChanged: updateMealSource,
                                    ),
                                  ),
                                  IconButton(
                                    onPressed: () {
                                      Navigator.push(
                                        context,
                                        MaterialPageRoute(
                                          builder: (context) =>
                                              _mealSource == null
                                                  ? const MealSourceDetailScreen()
                                                  : MealSourceDetailScreen(
                                                      id: _mealSource!.id),
                                        ),
                                      ).then((result) {
                                        updateMealSource(result?[1]);
                                        reload(message: result?[0]);
                                      });
                                    },
                                    icon: Icon(_mealSource == null
                                        ? Icons.add
                                        : Icons.edit),
                                  ),
                                ],
                              ),
                            ),
                            Padding(
                              padding: const EdgeInsets.symmetric(vertical: 5.0),
                              child: Row(
                                children: [
                                  Expanded(
                                    child:
                                        AutoCompleteDropdownButton<MealCategory>(
                                      controller: _mealCategoryController,
                                      selectedItem: _mealCategory,
                                      label: 'Meal Category',
                                      items: _mealCategories,
                                      onChanged: updateMealCategory,
                                    ),
                                  ),
                                  IconButton(
                                    onPressed: () {
                                      Navigator.push(
                                        context,
                                        MaterialPageRoute(
                                          builder: (context) => _mealCategory ==
                                                  null
                                              ? const MealCategoryDetailScreen()
                                              : MealCategoryDetailScreen(
                                                  id: _mealCategory!.id),
                                        ),
                                      ).then((result) {
                                        updateMealCategory(result?[1]);
                                        reload(message: result?[0]);
                                      });
                                    },
                                    icon: Icon(_mealCategory == null
                                        ? Icons.add
                                        : Icons.edit),
                                  ),
                                ],
                              ),
                            ),
                            Padding(
                              padding: const EdgeInsets.symmetric(vertical: 5.0),
                              child: Row(
                                children: [
                                  Expanded(
                                    child: AutoCompleteDropdownButton<
                                        MealPortionType>(
                                      controller: _mealPortionTypeController,
                                      selectedItem: _mealPortionType,
                                      label: 'Meal Portion Type',
                                      items: _mealPortionTypes,
                                      onChanged: updateMealPortionType,
                                    ),
                                  ),
                                  IconButton(
                                    onPressed: () {
                                      Navigator.push(
                                        context,
                                        MaterialPageRoute(
                                          builder: (context) => _mealPortionType ==
                                                  null
                                              ? const MealPortionTypeDetailScreen()
                                              : MealPortionTypeDetailScreen(
                                                  id: _mealPortionType!.id),
                                        ),
                                      ).then((result) {
                                        updateMealPortionType(result?[1]);
                                        reload(message: result?[0]);
                                      });
                                    },
                                    icon: Icon(_mealPortionType == null
                                        ? Icons.add
                                        : Icons.edit),
                                  ),
                                ],
                              ),
                            ),
                            Padding(
                              padding: const EdgeInsets.symmetric(vertical: 5.0),
                              child: Row(
                                children: [
                                  Expanded(
                                    child: AutoCompleteDropdownButton<Accuracy>(
                                      controller: _portionSizeAccuracyController,
                                      selectedItem: _portionSizeAccuracy,
                                      label: 'Portion Size Accuracy',
                                      items: _portionSizeAccuracies,
                                      onChanged: updatePortionSizeAccuracy,
                                    ),
                                  ),
                                  IconButton(
                                    onPressed: () {
                                      Navigator.push(
                                        context,
                                        MaterialPageRoute(
                                          builder: (context) => _portionSizeAccuracy == null
                                              ? const AccuracyDetailScreen()
                                              : AccuracyDetailScreen(
                                                  id: _portionSizeAccuracy!.id),
                                        ),
                                      ).then((result) {
                                        updatePortionSizeAccuracy(result?[1]);
                                        reload(message: result?[0]);
                                      });
                                    },
                                    icon: Icon(_portionSizeAccuracy == null
                                        ? Icons.add
                                        : Icons.edit),
                                  ),
                                ],
                              ),
                            ),
                            Padding(
                              padding: const EdgeInsets.symmetric(vertical: 5.0),
                              child: Row(
                                children: [
                                  Expanded(
                                    child: AutoCompleteDropdownButton<Accuracy>(
                                      controller: _carbsRatioAccuracyController,
                                      selectedItem: _carbsRatioAccuracy,
                                      label: 'Carbs Ratio Accuracy',
                                      items: _carbsRatioAccuracies,
                                      onChanged: updateCarbsRatioAccuracy,
                                    ),
                                  ),
                                  IconButton(
                                    onPressed: () {
                                      Navigator.push(
                                        context,
                                        MaterialPageRoute(
                                          builder: (context) => _carbsRatioAccuracy == null
                                              ? const AccuracyDetailScreen()
                                              : AccuracyDetailScreen(
                                                  id: _carbsRatioAccuracy!.id),
                                        ),
                                      ).then((result) {
                                        updateCarbsRatioAccuracy(result?[1]);
                                        reload(message: result?[0]);
                                      });
                                    },
                                    icon: Icon(_carbsRatioAccuracy == null
                                        ? Icons.add
                                        : Icons.edit),
                                  ),
                                ],
                              ),
                            ),
                          ]
                        : [],
                  ),
                ],
              ),
            ],
          ),
        ),
      ),
      bottomNavigationBar: DetailBottomRow(
        onCancel: handleCancelAction,
        onSave: _isSaving ? null : handleSaveAction,
      ),
    );
  }
}
